<div>
<b>GRAPH</b>
</div>
<script>
    var plugin = {
        refresh: function (table) {
            console.log('graph')
        }
    }
    django_datatables.add_plugin('{{ datatable.table_id }}', plugin)
</script>