.tag-filter {
    font-size: 12px;
    padding: 0 2px 0 2px;
    margin: 1px;
    font-family: Arial;
    display: inline-block;
    user-select: none;
    border-radius: .25rem;
}

.tag-filter:hover {
    width: 62px;
}

.option-3 {
    color: #fff;
    background-color: #28a745;
}

.option-3:after {
    content: '\2713';
}

.option-3:hover:after {
    content: '\2713 Required';
}

.option-2 {
    color: #fff;
    background-color: #007bff;
}

.option-2::after {
    content: '\2714';
}

.option-2:hover:after {
    content: '\2714 \00a0 Include';
}

.option-4 {
    color: #212529;
    background-color: #ffc107;
}

.option-4:after {
    content: '\2716';
}

.option-4:hover:after {
    content: '\2716 \00a0 Exclude';
}

.option-1 {
    color: #fff;
    background-color: #6c757d;
}

.option-1:after {
    content: '\2715';
}

.option-1:hover:after {
    content: '\2715 \00a0 Not set';
}

[data-toggle="collapse"] .expandicon::before {
    content: "\f102";
    font-family: "Font Awesome 5 Free", "Font Awesome 5 Pro";
    font-style: normal;
    font-weight: 800;
    text-decoration: inherit;
    display: inline-block;
    font-style: normal;
    font-variant: normal;
    text-rendering: auto;
    -webkit-font-smoothing: antialiased;
}

[data-toggle="collapse"].collapsed .expandicon:before {
    content: "\f103";
    font-family: "Font Awesome 5 Free", "Font Awesome 5 Pro";
    font-style: normal;
    font-weight: 800;
    text-decoration: inherit;
    display: inline-block;
    font-style: normal;
    font-variant: normal;
    text-rendering: auto;
    -webkit-font-smoothing: antialiased;
}

.section-header-right {
    float: right;
}

.filter-none .filter-header {
    border: 2px solid #dc354585;
    margin: -2px -2px -1px -2px;
}

.filter-active .filter-header {
    border: 2px solid #28a74554;
    margin: -2px -2px -1px -2px;
}

.filter-header-icon {
    font-size: large;
    color: #303030d6;
    text-decoration: none;
}

.filter-header-icon:hover {
    text-decoration: none;
}

.filter-all .all-check, .filter-none .none-check {
    color: darkgray;
    pointer-events: none;
    cursor: not-allowed;
}

.row_select{
    width: 6.5em;
    padding: .1rem;
    font-size: 0.8rem;
    line-height: 1;
    color: #495057;
    background-color: #fff;
    background-clip: padding-box;
    border: 1px solid #ced4da;
    border-radius: .25rem;
    transition: border-color .15s ease-in-out,box-shadow .15s ease-in-out;
}