<div class="w-100 float-left">
    <div class="form-check float-left">
        <label class="form-check-label small">
            <input data-value="%6" type="checkbox" class="form-check-input filtercheck" checked>%1
        </label>
    </div>
    <span data-value="%6" class="small badge badge-pill badge-primary float-right">-</span>
</div>