<tr>
    <td>
        <div class="form-check float-left">
            <label class="form-check-label">
                <input data-value="%6" type="checkbox" class="form-check-input filtercheck"
                       checked>%1</label>
        </div>
    </td>
    <td class="pr-1 text-right text-secondary total" data-value="%6"></td>
    <td class="text-right overall" data-value="%6"></td>
</tr>