<div class="tag-container">
    <div class="float-right">
        <span data-value="%4" class="small badge badge-pill badge-primary">0</span>
    </div>
    <span class="form-check-label small">
        <span data-value="%4" class="rounded tag-filter option-1"></span>
        %1
    </span>
</div>